# Generated by Django 4.2.8 on 2023-12-22 05:20

from django.db import migrations, models
import django.db.models.deletion
import django.utils.timezone
import model_utils.fields


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='ApiHistory',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True, null=True, verbose_name='생성일자')),
                ('updated_at', models.DateTimeField(auto_now=True, null=True, verbose_name='수정일자')),
                ('title', models.TextField(blank=True, null=True, verbose_name='제목')),
                ('method', models.TextField(blank=True, null=True, verbose_name='Method')),
                ('url', models.TextField(blank=True, null=True, verbose_name='URL')),
                ('headers', models.JSONField(blank=True, null=True, verbose_name='Header')),
                ('request', models.JSONField(blank=True, null=True, verbose_name='Request')),
                ('response', models.JSONField(blank=True, null=True, verbose_name='Response')),
                ('error', models.TextField(blank=True, null=True, verbose_name='Error')),
            ],
            options={
                'verbose_name': 'history',
                'verbose_name_plural': 'histories',
                'ordering': ['-created_at'],
            },
        ),
        migrations.CreateModel(
            name='NFT',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('status', model_utils.fields.StatusField(choices=[('PENDING', 'PENDING'), ('APPROVED', 'APPROVED')], default='PENDING', max_length=100, no_check_for_status=True, verbose_name='status')),
                ('status_changed', model_utils.fields.MonitorField(default=django.utils.timezone.now, monitor='status', verbose_name='status changed')),
                ('created_at', models.DateTimeField(auto_now_add=True, null=True, verbose_name='생성일자')),
                ('updated_at', models.DateTimeField(auto_now=True, null=True, verbose_name='수정일자')),
                ('token_id', models.IntegerField(null=True, unique=True, verbose_name='token_id')),
                ('request_id', models.UUIDField(null=True, unique=True, verbose_name='request_id')),
                ('name', models.TextField(blank=True, null=True, verbose_name='name')),
                ('address', models.TextField(blank=True, null=True, verbose_name='address')),
                ('nft_type', models.TextField(blank=True, null=True, verbose_name='nft_type')),
                ('attributes', models.TextField(blank=True, null=True, verbose_name='attributes')),
                ('image_url', models.URLField(blank=True, null=True, verbose_name='이미지 URL')),
                ('animation_url', models.URLField(blank=True, null=True, verbose_name='영상 URL')),
                ('external_url', models.URLField(blank=True, null=True, verbose_name='외부 URL')),
                ('extra_url', models.JSONField(blank=True, null=True, verbose_name='추가 URL')),
            ],
            options={
                'verbose_name': 'nft',
                'verbose_name_plural': 'nfts',
                'ordering': ['-created_at'],
            },
        ),
        migrations.CreateModel(
            name='NFTHistory',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True, null=True, verbose_name='생성일자')),
                ('updated_at', models.DateTimeField(auto_now=True, null=True, verbose_name='수정일자')),
                ('type', models.CharField(choices=[('CREATE', 'CREATE'), ('UPDATE', 'UPDATE'), ('TRANSFER', 'TRANSFER')], max_length=100, verbose_name='유형')),
                ('title', models.TextField(blank=True, null=True, verbose_name='제목')),
                ('nft', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='histories', to='creta.nft')),
            ],
            options={
                'verbose_name': 'nft_history',
                'verbose_name_plural': 'nft_histories',
                'ordering': ['-created_at'],
            },
        ),
    ]
